"""
scale_fc
========
Autogenerated DPF operator classes.
"""

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class scale_fc(Operator):
    """Scales a fields container by a constant factor. This factor can be a
    scalar, a vector a field, a fields container, where each value of
    the vector represents a scaler per component. Number of the
    components are corresponding to the input field dimensionality

    Parameters
    ----------
    fields_container : FieldsContainer
        Fields container to be scaled
    ponderation : float or Field or FieldsContainer
        Double/vector of
        doubles/field/fieldscontainer. when
        scoped on overall, same value(s)
        applied on all the data, when scoped
        elsewhere, corresponding values will
        be multiplied due to the scoping
    boolean : bool, optional
        Default is false. if set to true, output of
        scale is made dimensionless
    algorithm : int, optional
        Default is 0 use mkl. if set to 1, don't

    Returns
    -------
    fields_container : FieldsContainer

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.math.scale_fc()

    >>> # Make input connections
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_ponderation = float()
    >>> op.inputs.ponderation.connect(my_ponderation)
    >>> my_boolean = bool()
    >>> op.inputs.boolean.connect(my_boolean)
    >>> my_algorithm = int()
    >>> op.inputs.algorithm.connect(my_algorithm)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.math.scale_fc(
    ...     fields_container=my_fields_container,
    ...     ponderation=my_ponderation,
    ...     boolean=my_boolean,
    ...     algorithm=my_algorithm,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self,
        fields_container=None,
        ponderation=None,
        boolean=None,
        algorithm=None,
        config=None,
        server=None,
    ):
        super().__init__(name="scale_fc", config=config, server=server)
        self._inputs = InputsScaleFc(self)
        self._outputs = OutputsScaleFc(self)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if ponderation is not None:
            self.inputs.ponderation.connect(ponderation)
        if boolean is not None:
            self.inputs.boolean.connect(boolean)
        if algorithm is not None:
            self.inputs.algorithm.connect(algorithm)

    @staticmethod
    def _spec():
        description = """Scales a fields container by a constant factor. This factor can be a
            scalar, a vector a field, a fields container, where each
            value of the vector represents a scaler per component.
            Number of the components are corresponding to the input
            field dimensionality"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fields container to be scaled""",
                ),
                1: PinSpecification(
                    name="ponderation",
                    type_names=[
                        "double",
                        "field",
                        "vector<double>",
                        "fields_container",
                    ],
                    optional=False,
                    document="""Double/vector of
        doubles/field/fieldscontainer. when
        scoped on overall, same value(s)
        applied on all the data, when scoped
        elsewhere, corresponding values will
        be multiplied due to the scoping""",
                ),
                2: PinSpecification(
                    name="boolean",
                    type_names=["bool"],
                    optional=True,
                    document="""Default is false. if set to true, output of
        scale is made dimensionless""",
                ),
                3: PinSpecification(
                    name="algorithm",
                    type_names=["int32"],
                    optional=True,
                    document="""Default is 0 use mkl. if set to 1, don't""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="scale_fc", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsScaleFc
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsScaleFc
        """
        return super().outputs


class InputsScaleFc(_Inputs):
    """Intermediate class used to connect user inputs to
    scale_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.scale_fc()
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_ponderation = float()
    >>> op.inputs.ponderation.connect(my_ponderation)
    >>> my_boolean = bool()
    >>> op.inputs.boolean.connect(my_boolean)
    >>> my_algorithm = int()
    >>> op.inputs.algorithm.connect(my_algorithm)
    """

    def __init__(self, op: Operator):
        super().__init__(scale_fc._spec().inputs, op)
        self._fields_container = Input(scale_fc._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._fields_container)
        self._ponderation = Input(scale_fc._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._ponderation)
        self._boolean = Input(scale_fc._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._boolean)
        self._algorithm = Input(scale_fc._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._algorithm)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator.

        Fields container to be scaled

        Parameters
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.scale_fc()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def ponderation(self):
        """Allows to connect ponderation input to the operator.

        Double/vector of
        doubles/field/fieldscontainer. when
        scoped on overall, same value(s)
        applied on all the data, when scoped
        elsewhere, corresponding values will
        be multiplied due to the scoping

        Parameters
        ----------
        my_ponderation : float or Field or FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.scale_fc()
        >>> op.inputs.ponderation.connect(my_ponderation)
        >>> # or
        >>> op.inputs.ponderation(my_ponderation)
        """
        return self._ponderation

    @property
    def boolean(self):
        """Allows to connect boolean input to the operator.

        Default is false. if set to true, output of
        scale is made dimensionless

        Parameters
        ----------
        my_boolean : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.scale_fc()
        >>> op.inputs.boolean.connect(my_boolean)
        >>> # or
        >>> op.inputs.boolean(my_boolean)
        """
        return self._boolean

    @property
    def algorithm(self):
        """Allows to connect algorithm input to the operator.

        Default is 0 use mkl. if set to 1, don't

        Parameters
        ----------
        my_algorithm : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.scale_fc()
        >>> op.inputs.algorithm.connect(my_algorithm)
        >>> # or
        >>> op.inputs.algorithm(my_algorithm)
        """
        return self._algorithm


class OutputsScaleFc(_Outputs):
    """Intermediate class used to get outputs from
    scale_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.scale_fc()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(scale_fc._spec().outputs, op)
        self._fields_container = Output(scale_fc._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.scale_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container
